Imports GeoFramework
Imports GeoFramework.IO
Imports GeoFramework.Gps
Imports GeoFramework.Gps.Nmea
Imports GeoFramework.Gps.Emulators

Public Class Form1

    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' GPS.NET provides the ability to quickly discover GPS devices on the
        ' local system.  This feature is known as "Automatic Device Discovery"
        ' and greatly simplifies the task of finding a GPS device to communicate
        ' with.  Developers can simply call the "Start" method of an interpreter,
        ' and GPS.NET will automatically locate the best GPS Device to work with.
        ' 
        ' During Device Discovery, several events are raised to indicate the
        ' progress of detection.  This example hooks into almost all of the events
        ' to give users feedback.
        AddHandler GeoFramework.IO.Devices.DeviceDetectionStarted, AddressOf Devices_DeviceDetectionStarted
        AddHandler GeoFramework.IO.Devices.DeviceDetected, AddressOf Devices_DeviceDetected
        AddHandler GeoFramework.IO.Devices.DeviceDetectionAttempted, AddressOf Devices_DeviceDetectionAttempted
        AddHandler GeoFramework.IO.Devices.DeviceDetectionCompleted, AddressOf Devices_DeviceDetectionCompleted
        AddHandler GeoFramework.IO.Devices.DeviceDetectionExceptionOccurred, AddressOf Devices_DeviceDetectionExceptionOccurred
        AddHandler GeoFramework.IO.Devices.DeviceFailure, AddressOf Devices_DeviceFailure
    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As ComponentModel.CancelEventArgs) Handles MyBase.Closing
        ' The Dispose method should also be called for any objects that 
        ' implement it.  This prevents memory leaks and can help the application
        ' shut down more quickly. 

        Dispose(True)

        ' With everything shut down, the application can now gracefully exit. 
        Application.Exit()
    End Sub

#Region "  GPS Device Detection Events  "

    Private Sub SearchButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles SearchButton.Click

        ' Start the GPS device detection process.  This operation will complete
        ' in the background, on a separate thread.  This method can be called again
        ' to attempt to locate new devices.
        GeoFramework.IO.Devices.Detect()
    End Sub

    Private Sub Devices_DeviceDetectionStarted(ByVal sender As Object, ByVal e As EventArgs)
        StatusBar1.Text = "Detecting GPS devices..."
        SearchButton.Enabled = False
    End Sub

    Private Sub Devices_DeviceFailure(ByVal sender As Object, ByVal e As GeoFramework.IO.DeviceEventArgs)
        StatusBar1.Text = "Not Responding: " + e.Device.ToString()
    End Sub

    Private Sub Devices_DeviceDetectionExceptionOccurred(ByVal sender As Object, ByVal e As GeoFramework.ExceptionEventArgs)
        StatusBar1.Text = "Error: " + e.Exception.ToString()
    End Sub

    Private Sub Devices_DeviceDetectionCompleted(ByVal sender As Object, ByVal e As EventArgs)
        StatusBar1.Text = "Device detection completed."
        SearchButton.Enabled = True
    End Sub

    Private Sub Devices_DeviceDetectionAttempted(ByVal sender As Object, ByVal e As GeoFramework.IO.DeviceEventArgs)
        StatusBar1.Text = "Detecting " + e.Device.ToString()
    End Sub

    Private Sub Devices_DeviceDetected(ByVal sender As Object, ByVal e As GeoFramework.IO.DeviceEventArgs)
        StatusBar1.Text = "Found Device: " + e.Device.ToString()

        ' Add it to the list box
        Dim DeviceItem As ListViewItem = New ListViewItem(e.Device.ToString())
        ' Associate the detected device with this list view item, so we can connect 
        ' to the device later on
        DeviceItem.Tag = e.Device
        ' Set the image of the new item
        DeviceItem.ImageIndex = 0
        DevicesListView.Items.Add(DeviceItem)
    End Sub

#End Region

#Region "  Connecting and Disconnecting  "

    Private Sub StartButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles StartButton.Click
        ' What radio button is selected?
        If UseAnyAvailableDeviceRadioButton.Checked Then
            ' "Use any available device"

            ' To instruct GPS.NET to use any available GPS device,
            ' the interpreter is assigned to a "GpsStream" object. This
            ' object provides data access to the fastest-responding GPS
            ' device on the system.
            Interpreter.BaseStream = New GpsStream()

        ElseIf (UseDeviceRadioButton.Checked) Then
            ' "Use the following GPS device:"

            ' GPS.NET interpreters can be assigned to any kind of
            ' Stream object, making them very versatile!  FileStream
            ' and NetworkStream objects can be used, as well as GPS.NET's
            ' own SerialStream and BluetoothStream classes.

            ' Find out which item is selected in the list view
            Dim SelectedItem As ListViewItem = DevicesListView.Items(DevicesListView.SelectedIndices(0))

            ' The "tag" property holds the device that was detected
            Dim SelectedDevice As Device = CType(SelectedItem.Tag, Device)

            ' Open a Stream to the GPS device and bind it to the interpreter
            Interpreter.BaseStream = SelectedDevice.GetHardwareStream()

        Else

            ' "Use a GPS emulator"

            ' GPS.NET includes a framework for designing your own
            ' GPS device emulator.  GPS.NET also includes some pre-made
            ' emulators which are very useful when building GPS 
            ' applications when no signal is present, such as in an
            ' office building.

            ' Create a new DeLorme Earthmate USB emulator
            Interpreter.BaseStream = New DeLormeEarthmateUsbEmulator()

        End If

        ' Finally, start the interpreter using the newly-assigned Stream
        Interpreter.Restart()

        ' Enable the disconnect button
        StopButton.Enabled = True
    End Sub

    Private Sub UseDeviceRadioButton_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles UseDeviceRadioButton.CheckedChanged

        ' Enable the device list view
        DevicesListView.Enabled = True
        DevicesListView.Focus()

    End Sub

    Private Sub UseAnyAvailableDeviceRadioButton_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles UseAnyAvailableDeviceRadioButton.CheckedChanged

        ' Enable the device list view
        StartButton.Enabled = True
        DevicesListView.Enabled = False
        StartButton.Focus()

    End Sub

    Private Sub UseEmulatorRadioButton_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles UseEmulatorRadioButton.CheckedChanged

        ' Enable the device list view
        StartButton.Enabled = True
        DevicesListView.Enabled = False
        StartButton.Focus()

    End Sub

    Private Sub StopButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles StopButton.Click

        ' Stop any GPS communications
        Interpreter.Stop()

        ' And disable the disconnect button
        StopButton.Enabled = False
        StatusBar1.Text = "Disconnected from GPS device."

    End Sub

    Private Sub DevicesListView_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) Handles DevicesListView.SelectedIndexChanged

        ' Enable the Connect button
        UseDeviceRadioButton.Checked = True
        StartButton.Enabled = True
        StartButton.Focus()

    End Sub

    Sub Interpreter_Disconnected(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.Disconnected

        StatusBar1.Text = "Disconnected."
        Cursor.Current = Cursors.Default

    End Sub

    Sub Interpreter_Disconnecting(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.Disconnecting

        StatusBar1.Text = "Disconnecting from GPS device..."
        Cursor.Current = Cursors.WaitCursor

    End Sub

    Sub Interpreter_Connected(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.Connected

        StatusBar1.Text = "Connected to GPS device!  Reading data..."
        Cursor.Current = Cursors.Default

    End Sub

    Sub Interpreter_Connecting(ByVal sender As Object, ByVal e As CancelableEventArgs) Handles Interpreter.Connecting

        Cursor.Current = Cursors.WaitCursor
        StatusBar1.Text = "Connecting to GPS device..."

    End Sub

    Sub Interpreter_TimeoutOccurred(ByVal sender As Object, ByVal e As ExceptionEventArgs) Handles Interpreter.TimeoutOccurred

        StatusBar1.Text = "The device stopped.  Restarting..."

    End Sub

#End Region

#Region "  Working with Raw Data  "

    Private Sub Interpreter_SentenceReceived(ByVal sender As Object, ByVal e As GeoFramework.Gps.Nmea.NmeaSentenceEventArgs) Handles Interpreter.SentenceReceived

        ' Keep the list box from getting too big
        If RawDataListBox.Items.Count > 50 Then
            RawDataListBox.Items.RemoveAt(0)
        End If

        ' Add the sentence to the raw data list box
        RawDataListBox.Items.Add(e.Sentence.ToString())
        RawDataListBox.SelectedIndex = RawDataListBox.Items.Count - 1

    End Sub

#End Region

#Region "  Working with the Current Position  "

    Private Sub Interpreter_PositionChanged(ByVal sender As Object, ByVal e As GeoFramework.PositionEventArgs) Handles Interpreter.PositionChanged

        ' Update the latitude and longitude
        Latitude.Text = e.Position.Latitude.ToString()
        Longitude.Text = e.Position.Longitude.ToString()

        ' For international developers, the Position class also
        ' stores UTM coordinate information.  See the Zone,
        ' Easting and Northing properties for more information
        '
        'Latitude.Text = e.Position.Northing.ToString()
        'Longitude.Text = e.Position.Easting.ToString()

        StatusBar1.Text = "Position has changed."

    End Sub

#End Region

#Region "  Working with the Current Altitude  "

    Private Sub Interpreter_AltitudeChanged(ByVal sender As Object, ByVal e As GeoFramework.DistanceEventArgs) Handles Interpreter.AltitudeChanged

        ' Update the current altitude
        Altitude.Text = e.Distance.ToLocalUnitType().ToString()
        StatusBar1.Text = "Altitude has changed."

    End Sub

#End Region

#Region "  Working with the Current Speed  "

    Private Sub Interpreter_SpeedChanged(ByVal sender As Object, ByVal e As GeoFramework.SpeedEventArgs) Handles Interpreter.SpeedChanged

        ' Update the current speed
        Speed.Text = e.Speed.ToLocalUnitType().ToString()
        StatusBar1.Text = "Speed has changed."

    End Sub

#End Region

#Region "  Working with Fix Obtained/Lost Events  "

    Private Sub Interpreter_FixObtained(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.FixObtained

        ' Change the fix status            
        FixStatus.Text = "Fix Obtained!"
        StatusBar1.Text = "A fix has been obtained!"

    End Sub

    Private Sub Interpreter_FixLost(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.FixLost

        ' Change the fix status
        FixStatus.Text = "Fix Lost"
        StatusBar1.Text = "The current fix was lost."

    End Sub

#End Region

#Region "  Working with the Current Bearing  "

    Private Sub Interpreter_BearingChanged(ByVal sender As Object, ByVal e As GeoFramework.AzimuthEventArgs) Handles Interpreter.BearingChanged

        ' Output the current bearing as degrees (i.e. 012)
        Bearing.Text = e.Azimuth.ToString()
        ' Output the current bearing as a compass direction (i.e. Southwest)
        Direction.Text = e.Azimuth.ToString("c")
        StatusBar1.Text = "Bearing has changed."

    End Sub

#End Region

#Region "  Working with the Current Fix Mode  "

    Private Sub Interpreter_FixModeChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.FixModeEventArgs) Handles Interpreter.FixModeChanged

        ' Update the current fix mode
        FixMode.Text = e.FixMode.ToString()
        StatusBar1.Text = "Fix mode has changed."

    End Sub

#End Region

#Region "  Working with the Current Fix Quality  "

    Private Sub Interpreter_FixQualityChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.FixQualityEventArgs) Handles Interpreter.FixQualityChanged

        ' Update the current fix quality
        FixQuality.Text = e.FixQuality.ToString()
        StatusBar1.Text = "Fix quality has changed."

    End Sub

#End Region

#Region "  Working with Satellite-Derived Time  "

    Private Sub Interpreter_UtcDateTimeChanged(ByVal sender As Object, ByVal e As GeoFramework.DateTimeEventArgs) Handles Interpreter.UtcDateTimeChanged

        ' Update the current satellite-derived time
        SatelliteTime.Text = e.DateTime.ToLocalTime().ToString()

    End Sub

    Private Sub LocalTimeTimer_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles LocalTimeTimer.Tick

        LocalTime.Text = DateTime.Now.ToString()

    End Sub
#End Region

#Region "  Enforcing GPS Precision  "

    Private Sub Interpreter_HorizontalDilutionOfPrecisionChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.DilutionOfPrecisionEventArgs) Handles Interpreter.HorizontalDilutionOfPrecisionChanged

        ' Update the horizontal DOP on the track bar
        CurrentHdopTrackBar.Value = CType(e.DilutionOfPrecision.Value, Integer)
        UpdateDopStatus()
        StatusBar1.Text = "Lat/long precision has changed."

    End Sub

    Private Sub Interpreter_VerticalDilutionOfPrecisionChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.DilutionOfPrecisionEventArgs) Handles Interpreter.VerticalDilutionOfPrecisionChanged

        ' Update the vertical DOP on the track bar
        CurrentVdopTrackBar.Value = CType(e.DilutionOfPrecision.Value, Integer)
        UpdateDopStatus()
        StatusBar1.Text = "Altitude precision has changed."

    End Sub

    Private Sub UpdateDopStatus()

        ' Are readings now precise enough to use?
        If (CurrentHdopTrackBar.Value <= MaximumHdopTrackBar.Value) Then
            HdopStatusLabel.Text = "Position readings are being accepted."
        Else
            HdopStatusLabel.Text = "Position readings are being ignored."

            ' Are readings now precise enough to use?
            If (CurrentVdopTrackBar.Value <= MaximumVdopTrackBar.Value) Then
                VdopStatusLabel.Text = "Altitude readings are being accepted."
            Else
                VdopStatusLabel.Text = "Altitude readings are being ignored."
            End If
        End If

    End Sub

    Private Sub MaximumHdopTrackBar_ValueChanged(ByVal sender As Object, ByVal e As EventArgs) Handles MaximumHdopTrackBar.ValueChanged

        ' Set the new maximum latitude/longitude precision error
        Interpreter.MaximumHorizontalDilutionOfPrecision = _
                New DilutionOfPrecision(MaximumHdopTrackBar.Value)
        ' And update the UI
        UpdateDopStatus()

    End Sub

    Private Sub MaximumVdopTrackBar_ValueChanged(ByVal sender As Object, ByVal e As EventArgs) Handles MaximumVdopTrackBar.ValueChanged

        ' Set the new maximum altitude precision error
        Interpreter.MaximumVerticalDilutionOfPrecision = _
                New DilutionOfPrecision(MaximumVdopTrackBar.Value)
        ' And update the UI
        UpdateDopStatus()

    End Sub

#End Region

#Region "  Handling PocketPC Power Suspend & Resume Events"

    Sub Interpreter_DeviceResumed(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.DeviceResumed

        Cursor.Current = Cursors.Default
        StatusBar1.Text = "Device is no longer suspended!  Restarting..."

    End Sub

    Sub Interpreter_DeviceSuspended(ByVal sender As Object, ByVal e As EventArgs) Handles Interpreter.DeviceSuspended

        StatusBar1.Text = "Device was powered off!"
        Cursor.Current = Cursors.WaitCursor

    End Sub

#End Region

End Class
