using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GeoFramework.Examples
{
    public partial class LiveDataForm : Form
    {
        public LiveDataForm()
        {
            InitializeComponent();

            // Start receiving live data
            GpsDevice.Start();
        }

        private void GpsDevice_PositionChanged(object sender, PositionEventArgs e)
        {
            // Update the form with the current position
            Latitude.Text = e.Position.Latitude.ToString();
            Longitude.Text = e.Position.Longitude.ToString();
            UtmZone.Text = e.Position.Zone.ToString();
            UtmEasting.Text = e.Position.Easting.ToString();
            UtmNorthing.Text = e.Position.Northing.ToString();
        }

        private void GpsDevice_BearingChanged(object sender, GeoFramework.AzimuthEventArgs e)
        {
            // Update the form with the current bearing
            Bearing.Text = e.Azimuth.ToString("h.hhh");
            DirectionShort.Text = e.Azimuth.ToString("c");
            DirectionLong.Text = e.Azimuth.ToString("cc");
        }

        private void GpsDevice_SpeedChanged(object sender, GeoFramework.SpeedEventArgs e)
        {
            // Update the form with the current speed
            SpeedKPH.Text = e.Speed.ToKilometersPerHour().ToString();
            SpeedMPH.Text = e.Speed.ToStatuteMilesPerHour().ToString();
        }

        private void GpsDevice_AltitudeChanged(object sender, GeoFramework.DistanceEventArgs e)
        {
            // Update the form with the current altitude
            AltitudeMeters.Text = e.Distance.ToMeters().ToString();
            AltitudeFeet.Text = e.Distance.ToFeet().ToString();
        }

        private void GpsDevice_HorizontalDilutionOfPrecisionChanged(object sender, GeoFramework.Gps.DilutionOfPrecisionEventArgs e)
        {
            // Update the form with the current lat/long accuracy
            HorizontalDilutionOfPrecision.Text = e.DilutionOfPrecision.ToString();
            // Also show the estimated accuracy 
            EstimatedHorizontalAccuracy.Text = e.DilutionOfPrecision.GetEstimatedError().ToLocalUnitType().ToString();
        }

        private void GpsDevice_MeanDilutionOfPrecisionChanged(object sender, GeoFramework.Gps.DilutionOfPrecisionEventArgs e)
        {
            // Update the form with the current overall precision
            MeanDilutionOfPrecision.Text = e.DilutionOfPrecision.ToString();
        }

        private void GpsDevice_VerticalDilutionOfPrecisionChanged(object sender, GeoFramework.Gps.DilutionOfPrecisionEventArgs e)
        {
            // Update the form with the current altitude precision
            VerticalDilutionOfPrecision.Text = e.DilutionOfPrecision.ToString();
            // Also show the estimated accuracy
            EstimatedVerticalAccuracy.Text = e.DilutionOfPrecision.GetEstimatedError().ToLocalUnitType().ToString();
        }

        private void GpsDevice_FixLost(object sender, System.EventArgs e)
        {
            // Update the form, showing that the fix was lost
            FixStatus.Text = "No Fix";
        }

        private void GpsDevice_FixObtained(object sender, System.EventArgs e)
        {
            // Update the form, showing that a fix is now obtained
            FixStatus.Text = "Fix Obtained";
        }

        private void GpsDevice_FixMethodChanged(object sender, GeoFramework.Gps.FixMethodEventArgs e)
        {
            // Update the form with the current fix method
            FixMethod.Text = e.FixMethod.ToString();
        }

        private void GpsDevice_FixModeChanged(object sender, GeoFramework.Gps.FixModeEventArgs e)
        {
            // Update the form with the current fix mode
            FixMode.Text = e.FixMode.ToString();
        }

        private void GpsDevice_FixQualityChanged(object sender, GeoFramework.Gps.FixQualityEventArgs e)
        {
            // Update the form with the current fix quality
            FixQuality.Text = e.FixQuality.ToString();
        }
    }
}