using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GeoFramework.Examples
{
    public partial class CalculateDistanceForm : Form
    {
        private Position NewYork = new Position("4047N 7358W");
        private Position London = new Position("5132N 5W");
        private Position Sydney = new Position("340S 151E");

        public CalculateDistanceForm()
        {
            InitializeComponent();

            // Start collecting the current location
            GpsDevice.Start();

            // Set default Start and Destination locations
            StartPositionTextBox.Text = NewYork.ToString();
            DestinationPositionTextBox.Text = London.ToString();
            Recalculate();
        }

        #region Radio Button Code

        private void StartPosition1_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to New York City, New York
            StartPositionTextBox.Text = NewYork.ToString();
            Recalculate();
        }

        private void StartPosition2_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to London, England
            StartPositionTextBox.Text = London.ToString();
            Recalculate();
        }

        private void StartPosition3_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to Sydney, Australia
            StartPositionTextBox.Text = Sydney.ToString();
            Recalculate();
        }

        private void StartPosition4_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to the current position
            StartPositionTextBox.Text = GpsDevice.Position.ToString();
            Recalculate();
        }

        private void DestinationPosition1_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to New York City, New York, USA
            DestinationPositionTextBox.Text = NewYork.ToString();
            Recalculate();
        }

        private void DestinationPosition2_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to London, England
            DestinationPositionTextBox.Text = London.ToString();
            Recalculate();
        }

        private void DestinationPosition3_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the destination position to Sydney, Australia
            DestinationPositionTextBox.Text = Sydney.ToString();
            Recalculate();
        }

        private void DestinationPosition4_CheckedChanged(object sender, System.EventArgs e)
        {
            // Set the start position to the current position
            DestinationPositionTextBox.Text = GpsDevice.Position.ToString();
            Recalculate();
        }

        #endregion

        private void Recalculate()
        {
            try
            {
                // Convert the string values into position objects
                Position Start = Position.Parse(StartPositionTextBox.Text);
                Position Destination = Position.Parse(DestinationPositionTextBox.Text);
                // Calculate the distance
                Distance Result = Start.DistanceTo(Destination);
                // Now format it depending on the radio buttons
                if (AutomaticRadioButton.Checked)
                    // Convert to the smallest unit in the computer's local culture
                    Result = Result.ToLocalUnitType();
                else if (InchesRadioButton.Checked)
                    // Convert to inches
                    Result = Result.ToInches();
                else if (FeetRadioButton.Checked)
                    // Convert to feet
                    Result = Result.ToFeet();
                else if (StatuteMilesRadioButton.Checked)
                    // Convert to statute miles
                    Result = Result.ToStatuteMiles();
                else if (CentimetersRadioButton.Checked)
                    // Convert to centimeters
                    Result = Result.ToCentimeters();
                else if (MetersRadioButton.Checked)
                    // Convert to meters
                    Result = Result.ToMeters();
                else if (KilometersRadioButton.Checked)
                    // Convert to kilometers
                    Result = Result.ToKilometers();
                else if (NauticalMilesRadioButton.Checked)
                    // Convert to nautical miles
                    Result = Result.ToNauticalMiles();
                // Finally, output as a string
                AnswerLabel.Text = Result.ToString();
            }
            catch (FormatException ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void RecalculateDistance(object sender, System.ComponentModel.CancelEventArgs e)
        {
            Recalculate();
        }

        private void RecalculateDistance(object sender, System.EventArgs e)
        {
            Recalculate();
        }
    }
}