Imports GeoFramework

Public Class LiveDataForm

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Start the GPS Device
        GpsDevice.Start()

    End Sub

    Private Sub GpsDevice_PositionChanged(ByVal sender As Object, ByVal e As PositionEventArgs) Handles GpsDevice.PositionChanged
        ' Update the form with the current position
        Latitude.Text = e.Position.Latitude.ToString()
        Longitude.Text = e.Position.Longitude.ToString()
        UtmZone.Text = e.Position.Zone.ToString()
        UtmEasting.Text = e.Position.Easting.ToString()
        UtmNorthing.Text = e.Position.Northing.ToString()
    End Sub

    Private Sub GpsDevice_BearingChanged(ByVal sender As Object, ByVal e As GeoFramework.AzimuthEventArgs) Handles GpsDevice.BearingChanged
        ' Update the form with the current bearing
        Bearing.Text = e.Azimuth.ToString("h.hhh")
        DirectionShort.Text = e.Azimuth.ToString("c")
        DirectionLong.Text = e.Azimuth.ToString("cc")
    End Sub

    Private Sub GpsDevice_SpeedChanged(ByVal sender As Object, ByVal e As GeoFramework.SpeedEventArgs) Handles GpsDevice.SpeedChanged
        ' Update the form with the current speed
        SpeedKPH.Text = e.Speed.ToKilometersPerHour().ToString()
        SpeedMPH.Text = e.Speed.ToStatuteMilesPerHour().ToString()
    End Sub

    Private Sub GpsDevice_AltitudeChanged(ByVal sender As Object, ByVal e As GeoFramework.DistanceEventArgs) Handles GpsDevice.AltitudeChanged
        ' Update the form with the current altitude
        AltitudeMeters.Text = e.Distance.ToMeters().ToString()
        AltitudeFeet.Text = e.Distance.ToFeet().ToString()
    End Sub

    Private Sub GpsDevice_HorizontalDilutionOfPrecisionChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.DilutionOfPrecisionEventArgs) Handles GpsDevice.HorizontalDilutionOfPrecisionChanged
        ' Update the form with the current lat/long accuracy
        HorizontalDilutionOfPrecision.Text = e.DilutionOfPrecision.ToString()
        ' Also show the estimated accuracy 
        EstimatedHorizontalAccuracy.Text = e.DilutionOfPrecision.GetEstimatedError().ToLocalUnitType().ToString()
    End Sub

    Private Sub GpsDevice_MeanDilutionOfPrecisionChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.DilutionOfPrecisionEventArgs) Handles GpsDevice.MeanDilutionOfPrecisionChanged
        ' Update the form with the current overall precision
        MeanDilutionOfPrecision.Text = e.DilutionOfPrecision.ToString()
    End Sub

    Private Sub GpsDevice_VerticalDilutionOfPrecisionChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.DilutionOfPrecisionEventArgs) Handles GpsDevice.VerticalDilutionOfPrecisionChanged
        ' Update the form with the current altitude precision
        VerticalDilutionOfPrecision.Text = e.DilutionOfPrecision.ToString()
        ' Also show the estimated accuracy
        EstimatedVerticalAccuracy.Text = e.DilutionOfPrecision.GetEstimatedError().ToLocalUnitType().ToString()
    End Sub

    Private Sub GpsDevice_FixLost(ByVal sender As Object, ByVal e As System.EventArgs) Handles GpsDevice.FixLost
        ' Update the form, showing that the fix was lost
        FixStatus.Text = "No Fix"
    End Sub

    Private Sub GpsDevice_FixObtained(ByVal sender As Object, ByVal e As System.EventArgs) Handles GpsDevice.FixObtained
        ' Update the form, showing that a fix is now obtained
        FixStatus.Text = "Fix Obtained"
    End Sub

    Private Sub GpsDevice_FixMethodChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.FixMethodEventArgs) Handles GpsDevice.FixMethodChanged
        ' Update the form with the current fix method
        FixMethod.Text = e.FixMethod.ToString()
    End Sub

    Private Sub GpsDevice_FixModeChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.FixModeEventArgs) Handles GpsDevice.FixModeChanged
        ' Update the form with the current fix mode
        FixMode.Text = e.FixMode.ToString()
    End Sub

    Private Sub GpsDevice_FixQualityChanged(ByVal sender As Object, ByVal e As GeoFramework.Gps.FixQualityEventArgs) Handles GpsDevice.FixQualityChanged
        ' Update the form with the current fix quality
        FixQuality.Text = e.FixQuality.ToString()
    End Sub
End Class