using System;
using System.IO;
using System.Threading;
using System.Net.Sockets;
using GeoFramework.IO;

namespace GeoFramework.IO.Bluetooth
{
	/// <summary>
	/// Summary description for BluetoothDevice.
	/// </summary>
	public sealed class BluetoothDevice : Device
	{
        // This is the service ID for the virtual serial port for all devices 
		internal static Guid VirtualSerialPortServiceGuid = new Guid("{00001101-0000-1000-8000-00805F9B34FB}");

		private BluetoothAddress pAddress;
		private Guid pServiceGuid;
		private string pName;

		public BluetoothDevice(BluetoothAddress address)
			: this(address, VirtualSerialPortServiceGuid)
		{
		}

		public BluetoothDevice(BluetoothAddress address, string name)
			: this(address, VirtualSerialPortServiceGuid, name)
		{
		}

		public BluetoothDevice(BluetoothAddress address, Guid serviceGuid)
			: this(address, serviceGuid, "Unknown Bluetooth Device")
		{
		}

		public BluetoothDevice(BluetoothAddress address, Guid serviceGuid, string name)
		{
			pName = name;
			pAddress = address;
			pServiceGuid = serviceGuid;			
		}

		public override Stream GetHardwareStream()
		{
			if(IsImmediateNeedStreamAvailable)
			{
				IsImmediateNeedStreamAvailable = false;
				return ImmediateNeedStream;
			}
			else
			{
				try
				{
					//BluetoothStream Result = 
					//Result.Open();
					return new BluetoothStream(pAddress, pServiceGuid);
				}
				catch(IOException)
				{
					return null;
				}
			}
		}

		/// <summary>Outputs the name of the device.</summary>
		public override string ToString()
		{
			if(pName == null)
			{
				return "Bluetooth Device at " + pAddress.ToString();
			}
			else
			{
				return pName; // + " at " + pAddress.ToString();
			}			
		}

		public string DeviceName
		{
			get
			{
				return pName;
			}
		}

		public BluetoothAddress Address
		{
			get
			{
				return pAddress;
			}
		}
		
		public Guid ServiceGuid
		{
			get
			{
				return pServiceGuid;
			}
		}
	}
}
