using System;
using System.IO;
using System.Text;

namespace GeoFramework.IO
{
    /// <summary>
    /// Represents results of an attempt to identify a device as a GPS device.
    /// </summary>
	public sealed class DeviceRecognitionResult
	{
        private bool pIsGpsDevice;
        private Stream pDeviceStream;

        public static readonly DeviceRecognitionResult NotRecognized = new DeviceRecognitionResult(false, null);

        public DeviceRecognitionResult(bool isGpsDevice, Stream deviceStream)
        {
            pIsGpsDevice = isGpsDevice;
            pDeviceStream = deviceStream;
        }

        /// <summary>
        /// Indicates whether the device was confirmed to be a GPS device.
        /// </summary>
        public bool IsGpsDevice
        {
            get
            {
                return pIsGpsDevice;
            }
        }

        /// <summary>
        /// Indicates the data stream identified as providing GPS data.
        /// </summary>
        public Stream DeviceStream
        {
            get
            {
            return pDeviceStream;
            }
        }
	}
}
