using System;
using System.Collections;
using System.Text;

namespace GeoFramework.Gps.Nmea
{
    /// <summary>
    /// Represents an initialization message transmitted by Hewlett-Packard HW6515 series devices.
    /// </summary>
    /// <remarks>The HW651x series of GPS devices has an array of problems associated with it, from
    /// displaying strange windows during connection attempts, to suddenly reporting that a high-
    /// precision environment is active while also reporting no fixed satellites.  To alleviate some
    /// of these problems, GPS.NET uses this sentence to indicate that the GPS device has reset itself.
    /// Currently, this class has no implementation other than being used to reset GPS.NET's variables
    /// in order to avoid inaccurate data from HW651x devices at run-time.</remarks>
	public class PglorSentence : NmeaSentence 
	{
		#region Constructors

		public PglorSentence(string sentence) 
            : base(sentence)
		{
		}

		public PglorSentence(byte[] bytes) 
            : this(ASCIIEncoding.ASCII.GetString(bytes, 0, bytes.Length))
		{
		}

        public PglorSentence()
        {
            // Build the sentence from the position information
            StringBuilder Sentence = new StringBuilder();
            Sentence.Append("$PGLOR,");
            // Append latitude
            // Set the sentence
            SetSentence(Sentence.ToString());
            SetChecksum();
        }

		#endregion

		#region Static Members

		/// <summary>Returns a new instance by interpreting the specified string.</summary>
        public static PglorSentence Parse(string sentence)
		{
            return new PglorSentence(sentence);
		}

		/// <summary>Represents a $GPGLL sentence with no values.</summary>
        public static readonly PglorSentence Empty = new PglorSentence();

		#endregion
	}
}
