using System;
using System.Globalization;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Threading;
using GeoFramework;
#if !PocketPC || DesignTime || Framework20
using System.ComponentModel;
#endif
#if PocketPC
using GeoFramework.Licensing;
#endif

namespace GeoFramework.Controls
{
	/// <summary>
	/// Represents a user control used to display the current direction of travel.
	/// </summary>
#if !PocketPC || DesignTime
	[ToolboxBitmap("GeoFramework.Controls.Compass.bmp")]
	[DefaultProperty("Value")]
#endif
#if Framework20
#if !PocketPC
    [ToolboxItem(true)]
#endif
    //[DesignTimeVisible(true)]
#endif
	[CLSCompliant(false)]
	public sealed class Compass : PolarControl
	{
#if !PocketPC
		private System.Threading.Thread InterpolationThread;
		private bool IsInterpolationActive;
		//private ManualResetEvent InterpolationThreadWaitHandle = new ManualResetEvent(false);
		private ManualResetEvent AnimationWaitHandle = new ManualResetEvent(false);
		private Interpolator ValueInterpolator = new Interpolator(15, InterpolationMethod.CubicEaseOut);
		private int InterpolationIndex;
#endif
#if PocketPC
        private Font pDirectionLabelFont = new Font("Tahoma", 8.0f, FontStyle.Bold);
        private Angle pDirectionLabelInterval = new Angle(45);
        private Angle pAngleLabelInterval = new Angle(0);
        private Font pAngleLabelFont = new Font("Tahoma", 6.0f, FontStyle.Regular);
        private Angle pMinorTickInterval = new Angle(5);
#if Framework20
		        private Pen pDirectionTickPen = new Pen(Color.Black, 3.0f);
#else
		private Pen pDirectionTickPen = new Pen(Color.Black);
#endif

#else
		private Font pDirectionLabelFont = new Font("Tahoma", 12.0f, FontStyle.Bold); 
		private Angle pDirectionLabelInterval = new Angle(45);
		private Angle pAngleLabelInterval = new Angle(30);
		private Font pAngleLabelFont = new Font("Tahoma", 8.0f, FontStyle.Regular); 
		private Angle pMinorTickInterval = new Angle(2);
		private Pen pDirectionTickPen = new Pen(Color.Black, 3.0f);
#endif

		private Azimuth pBearing = Azimuth.North;
		private Angle pMajorTickInterval = new Angle(15);
		private Pen pCenterPen = new Pen(Color.Gray);
		private Pen pMinorTickPen = new Pen(Color.Black);
		private Pen pMajorTickPen = new Pen(Color.Black);
		private SolidBrush pDirectionLabelBrush = new SolidBrush(Color.Black);
		private SolidBrush pAngleLabelBrush = new SolidBrush(Color.Black);
		private string pAngleLabelFormat = "h";
		
        private static PolarCoordinate[] NeedlePointsNorth = new PolarCoordinate[]
			{
				new PolarCoordinate(5, new Angle(270), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(270), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(275), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(280), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(285), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(290), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(295), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(300), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(305), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(310), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(315), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(320), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(325), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(330), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(335), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(55, new Angle(0), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(25), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(30), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(35), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(40), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(45), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(50), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(55), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(60), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(65), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(70), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(75), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(80), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(85), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(90), Azimuth.North, PolarCoordinateOrientation.Clockwise)
			};
		private static PolarCoordinate[] NeedlePointsSouth = new PolarCoordinate[]
			{
				new PolarCoordinate(5, new Angle(270), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(270), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(275), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(280), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(285), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(290), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(295), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(300), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(305), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(310), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(315), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(320), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(325), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(330), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(335), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(355), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(55, new Angle(0), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(50, new Angle(5), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(25), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(30), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(35), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(40), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(45), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(50), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(55), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(60), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(65), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(70), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(75), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(80), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(85), Azimuth.North, PolarCoordinateOrientation.Clockwise),
				new PolarCoordinate(8, new Angle(90), Azimuth.North, PolarCoordinateOrientation.Clockwise)
			};
		private SolidBrush pNorthNeedleBrush = new SolidBrush(Color.Red);
		private Pen pNorthNeedlePen = new Pen(Color.Black);
		private SolidBrush pSouthNeedleBrush = new SolidBrush(Color.White);
		private Pen pSouthNeedlePen = new Pen(Color.Black);
		private bool pIsUsingRealTimeData = false;
#if !PocketPC
		private SolidBrush pNeedleShadowBrush = new SolidBrush(Color.FromArgb(128, 0, 0, 0));
		private Size pNeedleShadowSize = new Size(5, 5);
#endif
#if (PocketPC && Framework20)
		private const int MaximumGracefulShutdownTime = 2000;
#elif !PocketPC
		private const int MaximumGracefulShutdownTime = 500;
#endif

		//[CLSCompliant(false)]
        public event AzimuthEventHandler ValueChanged;



		public Compass()
			: base("GeoFramework Multithreaded Compass Control (http://www.geoframeworks.com)")
		{
#if !PocketPC
			// Start the interpolation thread
			InterpolationThread = new Thread(new ThreadStart(InterpolationLoop));
			InterpolationThread.IsBackground = true;
			InterpolationThread.Name = "GeoFramework Compass Needle Animation Thread (http://www.geoframeworks.com)";
			IsInterpolationActive = true;
			InterpolationThread.Start();
			//InterpolationThreadWaitHandle.WaitOne();
#endif
            Orientation = PolarCoordinateOrientation.Clockwise;
			Origin = Azimuth.North;
			// The center is zero and edge is 100
			CenterR = 0;
			MaximumR = 100;

			// Now mirror the points to create the southern needle
			NeedlePointsSouth = new PolarCoordinate[38];
			for (int index = 0; index < NeedlePointsNorth.Length; index++)
			{
				NeedlePointsSouth[index] = NeedlePointsNorth[index].Rotate(180);
			}

//#if PocketPC && !Framework20 && !DesignTime
//            // Bind the global event for when the current bearing changes
//            GeoFramework.IO.Devices.CurrentBearingChanged += new AzimuthEventHandler(Devices_CurrentBearingChanged);
//#endif
        }

#if !PocketPC || Framework20
        //protected override void OnHandleCreated(EventArgs e)
        //{

        //    // Subscribe to events
        //    try
        //    {
        //        base.OnHandleCreated(e);

        //        // Only hook into events if we're at run-time.  Hooking events
        //        // at design-time can actually cause errors in the WF Designer.
        //        if (LicenseManager.CurrentContext.UsageMode == LicenseUsageMode.Runtime)
        //        {
        //            GeoFramework.IO.Devices.CurrentBearingChanged += new AzimuthEventHandler(Devices_CurrentBearingChanged);
        //        }
        //    }
        //    catch
        //    {
        //    }

        //}

        //protected override void OnHandleDestroyed(EventArgs e)
        //{
        //    try
        //    {
        //        // Only hook into events if we're at run-time.  Hooking events
        //        // at design-time can actually cause errors in the WF Designer.
        //        if (LicenseManager.CurrentContext.UsageMode == LicenseUsageMode.Runtime)
        //        {
        //            GeoFramework.IO.Devices.CurrentBearingChanged -= new AzimuthEventHandler(Devices_CurrentBearingChanged);
        //        }
        //    }
        //    catch
        //    {
        //    }
        //    finally
        //    {
        //        base.OnHandleDestroyed(e);
        //    }
        //}
#endif

        protected override void Dispose(bool disposing)
        {
            try
            {
                // Only hook into events if we're at run-time.  Hooking events
                // at design-time can actually cause errors in the WF Designer.
                if (LicenseManager.CurrentContext.UsageMode == LicenseUsageMode.Runtime
                    && pIsUsingRealTimeData)
                {
                    GeoFramework.IO.Devices.CurrentBearingChanged -= new AzimuthEventHandler(Devices_CurrentBearingChanged);
                }
            }
            catch
            {
            }

#if !PocketPC
			// Get the interpolation thread out of a loop 
			IsInterpolationActive = false;

            if (InterpolationThread != null)
            {

                if (AnimationWaitHandle != null)
                {
                    try
                    {
                        AnimationWaitHandle.Set();
                    }
                    catch
                    {
                    }
                }

                if (!InterpolationThread.Join(MaximumGracefulShutdownTime))
                {
                    try
                    {
                        InterpolationThread.Abort();
                    }
                    catch
                    {
                    }
                }

            }

			if (AnimationWaitHandle != null)
			{
				try
				{
					AnimationWaitHandle.Close();
				}
				catch
				{
				}
				finally
				{
					AnimationWaitHandle = null;
				}
			}
#endif

            if (pCenterPen != null)
            {
                try
                {
                    pCenterPen.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pCenterPen = null;
                }
            }
            if (pMinorTickPen != null)
            {
                try
                {
                    pMinorTickPen.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pMinorTickPen = null;
                }
            }
            if (pMajorTickPen != null)
            {
                try
                {
                    pMajorTickPen.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pMajorTickPen = null;
                }
            }
            if (pDirectionLabelFont != null)
            {
                try
                {
                    pDirectionLabelFont.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pDirectionLabelFont = null;
                }
            }
            if (pDirectionTickPen != null)
            {
                try
                {
                    pDirectionTickPen.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pDirectionTickPen = null;
                }
            }
            if (pAngleLabelFont != null)
            {
                try
                {
                    pAngleLabelFont.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pAngleLabelFont = null;
                }
            }
#if !PocketPC
			if (pNeedleShadowBrush != null)
			{
				pNeedleShadowBrush.Dispose();
				pNeedleShadowBrush = null;
			}
#endif
            if (pNorthNeedleBrush != null)
            {
                try
                {
                    pNorthNeedleBrush.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pNorthNeedleBrush = null;
                }
            }
            if (pNorthNeedlePen != null)
            {
                try
                {
                    pNorthNeedlePen.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pNorthNeedlePen = null;
                }
            }
            if (pSouthNeedleBrush != null)
            {
                try
                {
                    pSouthNeedleBrush.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pSouthNeedleBrush = null;
                }
            }
            if (pSouthNeedlePen != null)
            {
                try
                {
                    pSouthNeedlePen.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pSouthNeedlePen = null;
                }
            }
            if (pDirectionLabelBrush != null)
            {
                try
                {
                    pDirectionLabelBrush.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pDirectionLabelBrush = null;
                }
            }
            if (pAngleLabelBrush != null)
            {
                try
                {
                    pAngleLabelBrush.Dispose();
                }
                catch
                {
                }
                finally
                {
                    pAngleLabelBrush = null;
                }
            }

            // Move on down the line
            try
            {
                base.Dispose(disposing);
            }
            catch
            {
            }
        }

       

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Azimuth), "0")]
		[Category("Appearance")]
		[Description("Controls the direction that the needle points to.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Azimuth Value
		{
			get
			{
				return pBearing;
			}
			set
			{
				if(pBearing.Equals(value)) return;
				pBearing = value;
#if PocketPC
                InvokePaintOffScreen();
#else
				if(IsDisposed)
					return;

				lock(ValueInterpolator)
				{
					// Are we changing direction?
					if(pBearing.DecimalDegrees >= ValueInterpolator.Minimum
						&& pBearing.DecimalDegrees > ValueInterpolator[InterpolationIndex])
					{
						// No.  Just set the new maximum
						ValueInterpolator.Maximum = pBearing.DecimalDegrees;
					}					
					else if(pBearing.DecimalDegrees < ValueInterpolator.Minimum)
					{
						// We're changing directions, so stop then accellerate again
						ValueInterpolator.Minimum = ValueInterpolator[InterpolationIndex];
						ValueInterpolator.Maximum = pBearing.DecimalDegrees;
						InterpolationIndex = 0;
					}
					else if(pBearing.DecimalDegrees > ValueInterpolator.Minimum
						&& pBearing.DecimalDegrees < ValueInterpolator[InterpolationIndex])
					{
						// We're changing directions, so stop then accellerate again
						ValueInterpolator.Minimum = ValueInterpolator[InterpolationIndex];
						ValueInterpolator.Maximum = pBearing.DecimalDegrees;
						InterpolationIndex = 0;
					}
					else if(pBearing.DecimalDegrees > ValueInterpolator.Maximum)
					{
						// No.  Just set the new maximum
						ValueInterpolator.Maximum = pBearing.DecimalDegrees;
					}

					// If the difference is > 180, adjust so that it moves the right direction
					if(ValueInterpolator.Maximum - ValueInterpolator.Minimum > 180)
						ValueInterpolator.Minimum = ValueInterpolator.Minimum % 360.0 + 360.0;				
					else if(ValueInterpolator.Minimum - ValueInterpolator.Maximum > 180)
						ValueInterpolator.Maximum = ValueInterpolator.Maximum % 360.0 + 360.0;				
				}
				// And activate the interpolation thread
//				if((InterpolationThread.ThreadState & ThreadState.Suspended) != 0)
//					InterpolationThread.Resume();
				AnimationWaitHandle.Set();

#endif

                OnValueChanged(new AzimuthEventArgs(pBearing));
			}
		}

        private void OnValueChanged(AzimuthEventArgs e)
        {
            if (ValueChanged != null)
#if PocketPC && !Framework20
				ValueChanged(this, e);
#else
                DynamicEvent.BeginInvoke(ValueChanged, this, e);
#endif
        }

#if !PocketPC
		[Category("Behavior")]
		[DefaultValue(typeof(InterpolationMethod), "CubicEaseOut")]
		[Description("Controls how the control smoothly transitions from one value to another.")]
		//[CLSCompliant(false)]
		public InterpolationMethod ValueInterpolationMethod
		{
			get
			{
				return ValueInterpolator.InterpolationMethod;
			}
			set
			{
				ValueInterpolator.InterpolationMethod = value;
			}
		}
#endif

#if !PocketPC || DesignTime
#if PocketPC
		[DefaultValue(typeof(Angle), "0")]
#else
		[DefaultValue(typeof(Angle), "30.0")]
#endif
		[Category("Angle Labels")]
		[Description("Controls the number of degrees in between each label around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		//[CLSCompliant(false)]
		public Angle AngleLabelInterval
		{
			get
			{
				return pAngleLabelInterval;
			}
			set
			{
				if(pAngleLabelInterval.Equals(value)) return;
				pAngleLabelInterval = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Angle Labels")]
		[Description("Controls the color of degree labels drawn around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color AngleLabelColor
		{
			get
			{
				return pAngleLabelBrush.Color;
			}
			set
			{
				if(pAngleLabelBrush.Color.Equals(value)) return;
				pAngleLabelBrush.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
#if PocketPC
		[DefaultValue(typeof(Font), "Tahoma, 6pt")]
#else
		[DefaultValue(typeof(Font), "Tahoma, 8pt")]
#endif
		[Category("Angle Labels")]
		[Description("Controls the font of degree labels drawn around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Font AngleLabelFont
		{
			get
			{
				return pAngleLabelFont;
			}
			set
			{
				if(pAngleLabelFont.Equals(value)) return;
				pAngleLabelFont = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[Category("Direction Labels")]
		[DefaultValue(typeof(Angle), "45")]
		[Description("Controls the number of degrees in between each compass direction (i.e. \"N\", \"NW\") around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
			//[CLSCompliant(false)]
		public Angle DirectionLabelInterval
		{
			get
			{
				return pDirectionLabelInterval;
			}
			set
			{
				if(pDirectionLabelInterval.Equals(value)) return;
				pDirectionLabelInterval = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Direction Labels")]
		[Description("Controls the color of compass labels on the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color DirectionLabelColor
		{
			get
			{
				return pDirectionLabelBrush.Color;
			}
			set
			{
				if(pDirectionLabelBrush.Color.Equals(value)) return;
				pDirectionLabelBrush.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Direction Labels")]
		[Description("Controls the color of tick marks next to each direction label on the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color DirectionTickColor
		{
			get
			{
				return pDirectionTickPen.Color;
			}
			set
			{
				if(pDirectionTickPen.Color.Equals(value)) return;
				pDirectionTickPen.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[Category("Behavior")]
		[DefaultValue(typeof(bool), "False")]
		[Description("Controls whether the Value property is set manually, or automatically read from any available GPS device.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public bool IsUsingRealTimeData
		{
			get
			{
				return pIsUsingRealTimeData;
			}
			set
			{
                // Anything to do?
				if(pIsUsingRealTimeData == value) 
                    return;

                // Record the new value
				pIsUsingRealTimeData = value;

				// Set the control to the last known bearing (if any)
                if (pIsUsingRealTimeData)
                {
                    // Hook into real-time events.
                    // Only hook into events if we're at run-time.  Hooking events
                    // at design-time can actually cause errors in the WF Designer.
                    if (LicenseManager.CurrentContext.UsageMode == LicenseUsageMode.Runtime)
                    {
                        GeoFramework.IO.Devices.CurrentBearingChanged += new AzimuthEventHandler(Devices_CurrentBearingChanged);
                    }

                    // Set the current bearing
                    Value = GeoFramework.IO.Devices.CurrentBearing;
                }
                else
                {
                    // Only hook into events if we're at run-time.  Hooking events
                    // at design-time can actually cause errors in the WF Designer.
                    if (LicenseManager.CurrentContext.UsageMode == LicenseUsageMode.Runtime)
                    {
                        GeoFramework.IO.Devices.CurrentBearingChanged -= new AzimuthEventHandler(Devices_CurrentBearingChanged);
                    }

                    // Set a bearing of zero
                    Value = Azimuth.North;
                }

                // Tell the control to repaint
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
#if PocketPC
		[DefaultValue(typeof(Font), "Tahoma, 8pt, style=Bold")]
#else
		[DefaultValue(typeof(Font), "Tahoma, 12pt, style=Bold")]
#endif
		[Category("Direction Labels")]
		[Description("Controls the font used to draw direction labels on the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Font DirectionLabelFont
		{
			get
			{
				return pDirectionLabelFont;
			}
			set
			{
				if(pDirectionLabelFont.Equals(value)) return;
				pDirectionLabelFont = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(string), "h")]
		[Category("Angle Labels")]
		[Description("Controls the output format of labels drawn around the control. (i.e.  hm's\")")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public string AngleLabelFormat
		{
			get
			{
				return pAngleLabelFormat;
			}
			set
			{
				if(pAngleLabelFormat.Equals(value)) return;
				pAngleLabelFormat = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[Category("Tick Marks")]
		[Description("Controls the number of degrees in between each small tick mark around the control.")]
#if PocketPC
		[DefaultValue(typeof(Angle), "5")]
#else
		[DefaultValue(typeof(Angle), "2")]
#endif
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		//[CLSCompliant(false)]
		public Angle MinorTickInterval
		{
			get
			{
				return pMinorTickInterval;
			}
			set
			{
				if(pMinorTickInterval == value) return;
				pMinorTickInterval = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Tick Marks")]
		[Description("Controls the color of smaller tick marks drawn around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color MinorTickColor
		{
			get
			{
				return pMinorTickPen.Color;
			}
			set
			{
				pMinorTickPen.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Angle), "15")]
		[Category("Tick Marks")]
		[Description("Controls the number of degrees in between each larger tick mark around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
			//[CLSCompliant(false)]
		public Angle MajorTickInterval
		{
			get
			{
				return pMajorTickInterval;
			}
			set
			{
				if(pMajorTickInterval == value) return;
				pMajorTickInterval = value;
				InvokePaintOffScreen();
			}
		}

//#if Framework20 && !PocketPC
//        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
//        public override Azimuth Origin
//        {
//            get
//            {
//                return base.Origin;
//            }
//            set
//            {
//                base.Origin = value;
//            }
//        }        
//
//        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
//        public override Angle Rotation
//        {
//            get
//            {
//                return base.Rotation;
//            }
//            set
//            {
//                base.Rotation = value;
//            }
//        }
//#endif

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Tick Marks")]
		[Description("Controls the color of larger tick marks drawn around the control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color MajorTickColor
		{
			get
			{
				return pMajorTickPen.Color;
			}
			set
			{
				pMajorTickPen.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Red")]
		[Category("Compass Needle")]
		[Description("Controls the color of the interior of the needle which points North.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color NorthNeedleFillColor
		{
			get
			{
				return pNorthNeedleBrush.Color;
			}
			set
			{
				if(pNorthNeedleBrush.Color.Equals(value)) return;
				pNorthNeedleBrush.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Compass Needle")]
		[Description("Controls the color of the edge of the needle which points North.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color NorthNeedleOutlineColor
		{
			get
			{
				return pNorthNeedlePen.Color;
			}
			set
			{
				if(pNorthNeedlePen.Color.Equals(value)) return;
				pNorthNeedlePen.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "White")]
		[Category("Compass Needle")]
		[Description("Controls the color of the interior of the needle which points South.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color SouthNeedleFillColor
		{
			get
			{
				return pSouthNeedleBrush.Color;
			}
			set
			{
				if(pSouthNeedleBrush.Color.Equals(value)) return;
				pSouthNeedleBrush.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC || DesignTime
		[DefaultValue(typeof(Color), "Black")]
		[Category("Compass Needle")]
		[Description("Controls the color of the edge of the needle which points South.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
#endif
		public Color SouthNeedleOutlineColor
		{
			get
			{
				return pSouthNeedlePen.Color;
			}
			set
			{
				if(pSouthNeedlePen.Color.Equals(value)) return;
				pSouthNeedlePen.Color = value;
				InvokePaintOffScreen();
			}
		}

#if !PocketPC
		[DefaultValue(typeof(Color), "128, 0, 0, 0")]
		[Category("Appearance")]
		[Description("Controls the color of the shadow cast by the compass needle.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		public Color NeedleShadowColor
		{
			get
			{
				return pNeedleShadowBrush.Color;
			}
			set
			{
				if(pNeedleShadowBrush.Color.Equals(value)) return;
				pNeedleShadowBrush.Color = value;
				InvokePaintOffScreen();
			}
		}
#endif

#if !PocketPC
		[DefaultValue(typeof(Size), "5, 5")]
		[Category("Appearance")]
		[Description("Controls the size of the shadow cast by the compass needle.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
		[Browsable(true)]
		[EditorBrowsable(EditorBrowsableState.Always)]
		public Size NeedleShadowSize
		{
			get
			{
				return pNeedleShadowSize;
			}
			set
			{
				pNeedleShadowSize = value;
				InvokePaintOffScreen();
			}
		}
#endif

		protected override void OnPaintOffScreen(CancelablePaintEventArgs e)
		{
			PolarGraphics f = CreatePolarGraphics(e.Graphics);
			
			// What altitude are we drawing?
#if PocketPC
            Azimuth BearingToRender = pBearing;
#else
			Azimuth BearingToRender;
			try
			{
				BearingToRender = new Azimuth(ValueInterpolator[InterpolationIndex]);
			}
			catch
			{
				throw;
			}
#endif

            // Cache drawing options in order to prevent race conditions during
            // drawing! 
            double MinorInterval = pMinorTickInterval.DecimalDegrees;
            double MajorInterval = pMajorTickInterval.DecimalDegrees;
            double DirectionInterval = pDirectionLabelInterval.DecimalDegrees;
            double AngleInterval = pAngleLabelInterval.DecimalDegrees;


			// Draw tick marks
            if (MinorInterval > 0)
			{
                for (double angle = 0; angle < 360; angle += MinorInterval)
				{
					// And draw a line
					f.DrawLine(pMinorTickPen, new PolarCoordinate(98, angle), new PolarCoordinate(100, angle));
				}
			}
			// Draw tick marks
            if (MajorInterval > 0)
			{
                for (double angle = 0; angle < 360; angle += MajorInterval)
				{
					// And draw a line
					f.DrawLine(pMajorTickPen, new PolarCoordinate(95, angle), new PolarCoordinate(100, angle));
				}
			}
            if (DirectionInterval > 0)
			{
                for (double angle = 0; angle < 360; angle += DirectionInterval)
				{
					// And draw a line
					f.DrawLine(pDirectionTickPen, new PolarCoordinate(92, angle), new PolarCoordinate(100, angle));
				}
			}
            if (AngleInterval > 0)
			{
                for (double angle = 0; angle < 360; angle += AngleInterval)
				{
					// Get the coordinate of the line's start
					PolarCoordinate start = new PolarCoordinate(60, angle, Azimuth.North, PolarCoordinateOrientation.Clockwise);
#if PocketPC
					f.DrawCenteredString(((Angle)angle).ToString(pAngleLabelFormat, CultureInfo.CurrentCulture), pAngleLabelFont, pAngleLabelBrush, start);
#else
                    f.DrawRotatedString(((Angle)angle).ToString(pAngleLabelFormat, CultureInfo.CurrentCulture), pAngleLabelFont, pAngleLabelBrush, start);
#endif
				}
			}
            if (DirectionInterval > 0)
			{
                for (double angle = 0; angle < 360; angle += DirectionInterval)
				{
					// Get the coordinate of the line's start
					PolarCoordinate start = new PolarCoordinate(80, angle, Azimuth.North, PolarCoordinateOrientation.Clockwise);
#if PocketPC
                    f.DrawCenteredString(((Azimuth)angle).ToString("c", CultureInfo.CurrentCulture), pDirectionLabelFont, pDirectionLabelBrush, start);
#else
                    f.DrawRotatedString(((Azimuth)angle).ToString("c", CultureInfo.CurrentCulture), pDirectionLabelFont, pDirectionLabelBrush, start);
#endif
				}
			}

			// Draw an ellipse at the center
			f.DrawEllipse(pCenterPen, PolarCoordinate.Empty, 10);

			// Now draw the needle shadow			
			PolarCoordinate[] NeedleNorth = NeedlePointsNorth.Clone() as PolarCoordinate[];
			PolarCoordinate[] NeedleSouth = NeedlePointsSouth.Clone() as PolarCoordinate[];

			// Adjust the needle to the current bearing
			for(int index = 0; index < NeedleNorth.Length; index++)
			{
				NeedleNorth[index] = NeedleNorth[index].Rotate(BearingToRender.DecimalDegrees);
				NeedleSouth[index] = NeedleSouth[index].Rotate(BearingToRender.DecimalDegrees);
			}

#if !PocketPC
			// Now draw a shadow
			f.Graphics.TranslateTransform(pNeedleShadowSize.Width, pNeedleShadowSize.Height, MatrixOrder.Append);

			f.FillPolygon(pNeedleShadowBrush, NeedleNorth);
			f.FillPolygon(pNeedleShadowBrush, NeedleSouth);

			f.Graphics.ResetTransform();
#endif

			f.FillPolygon(pNorthNeedleBrush, NeedleNorth);
			f.DrawPolygon(pNorthNeedlePen, NeedleNorth);
			f.FillPolygon(pSouthNeedleBrush, NeedleSouth);
			f.DrawPolygon(pSouthNeedlePen, NeedleSouth);

		}

		private void Devices_CurrentBearingChanged(object sender, AzimuthEventArgs e)
		{
			if(pIsUsingRealTimeData)
				Value = e.Azimuth;
		}

#if !PocketPC
		protected override void OnTargetFrameRateChanged(int framesPerSecond)
		{
			base.OnTargetFrameRateChanged(framesPerSecond);
			// Recalculate our things
			ValueInterpolator.Count = framesPerSecond;
			// Adjust the index if it's outside of bounds
			if(InterpolationIndex > ValueInterpolator.Count - 1)
				InterpolationIndex = ValueInterpolator.Count - 1;
		}
//
//		protected override void OnHandleCreated(EventArgs e)
//		{
//			try
//			{
//			}
//			catch
//			{
//				throw;
//			}
//			finally
//			{
//				base.OnHandleCreated(e);
//			}
//
//		}
//
//		protected override void OnHandleDestroyed(EventArgs e)
//		{
//			try
//			{
//				// Get the interpolation thread out of a loop 
//				IsInterpolationActive = false;
//
//				if(InterpolationThread != null)
//				{
//					if((InterpolationThread.ThreadState & ThreadState.Suspended) != 0)
//						InterpolationThread.Resume();
//					while(!InterpolationThread.Join(1000))
//					{
//						if((InterpolationThread.ThreadState & ThreadState.AbortRequested) == 0)
//							InterpolationThread.Abort();
//					}
//				}
//			}
//			catch
//			{
//				throw;
//			}
//			finally
//			{
//				base.OnHandleDestroyed(e);
//			}
//		}

		private void InterpolationLoop()
		{
			// Flag that we're alive
			//InterpolationThreadWaitHandle.Set();
			// Are we at the end?
			while(IsInterpolationActive)
			{
				try
				{
					// Wait for interpolation to actually be needed
					AnimationWaitHandle.WaitOne();
					//InterpolationThread.Suspend();
					// If we're shutting down, just exit
					if (!IsInterpolationActive)
						break;
					// Keep updating interpolation until we're done
					while (IsInterpolationActive && InterpolationIndex < ValueInterpolator.Count)
					{
						// Render the next value
						InvokePaintOffScreen();
						InterpolationIndex++;
						// Wait for the next frame
						Thread.Sleep(1000 / ValueInterpolator.Count);
					}
					// Reset interpolation
					ValueInterpolator.Minimum = ValueInterpolator.Maximum;
					InterpolationIndex = 0;
					AnimationWaitHandle.Reset();
				}
				catch(ThreadAbortException)
				{
					// Just exit!
					break;
				}
				catch
				{					
				}
			}
			// Flag that we're alive
			//InterpolationThreadWaitHandle.Set();
		}
#endif
	}
}
